@@ -28,11 +28,12 @@ from airflow.jobs.base_job_runner import BaseJobRunner
 from airflow.jobs.job import perform_heartbeat
 from airflow.models.taskinstance import TaskReturnCode
 from airflow.stats import Stats
+from airflow.traces.tracer import Trace
 from airflow.utils import timezone
 from airflow.utils.log.file_task_handler import _set_task_deferred_context_var
 from airflow.utils.log.logging_mixin import LoggingMixin
 from airflow.utils.net import get_hostname
-from airflow.utils.platform import IS_WINDOWS
+from airflow.utils.platform import IS_WINDOWS, getuser
 from airflow.utils.session import NEW_SESSION, provide_session
 from airflow.utils.state import TaskInstanceState
 
@@ -110,6 +111,8 @@ class LocalTaskJobRunner(BaseJobRunner, LoggingMixin):
         self.terminating = False
 
         self._state_change_checks = 0
+        # time spend after task completed, but before it exited - used to measure listener execution time
+        self._overtime = 0.0
 
     def _execute(self) -> int | None:
         from airflow.task.task_runner import get_task_runner
@@ -182,40 +185,60 @@ class LocalTaskJobRunner(BaseJobRunner, LoggingMixin):
             # If LocalTaskJob receives SIGTERM, LocalTaskJob passes SIGTERM to _run_raw_task
             # If the state of task_instance is changed, LocalTaskJob sends SIGTERM to _run_raw_task
             while not self.terminating:
-                # Monitor the task to see if it's done. Wait in a syscall
-                # (`os.wait`) for as long as possible so we notice the
-                # subprocess finishing as quick as we can
-                max_wait_time = max(
-                    0,  # Make sure this value is never negative,
-                    min(
-                        (
-                            heartbeat_time_limit
-                            - (timezone.utcnow() - self.job.latest_heartbeat).total_seconds() * 0.75
+                with Trace.start_span(
+                    span_name="local_task_job_loop", component="LocalTaskJobRunner"
+                ) as span:
+                    # Monitor the task to see if it's done. Wait in a syscall
+                    # (`os.wait`) for as long as possible so we notice the
+                    # subprocess finishing as quick as we can
+                    max_wait_time = max(
+                        0,  # Make sure this value is never negative,
+                        min(
+                            (
+                                heartbeat_time_limit
+                                - (timezone.utcnow() - self.job.latest_heartbeat).total_seconds() * 0.75
+                            ),
+                            self.job.heartrate if self.job.heartrate is not None else heartbeat_time_limit,
                         ),
-                        self.job.heartrate if self.job.heartrate is not None else heartbeat_time_limit,
-                    ),
-                )
-
-                return_code = self.task_runner.return_code(timeout=max_wait_time)
-                if return_code is not None:
-                    self.handle_task_exit(return_code)
-                    return return_code
-
-                perform_heartbeat(
-                    job=self.job, heartbeat_callback=self.heartbeat_callback, only_if_necessary=False
-                )
-
-                # If it's been too long since we've heartbeat, then it's possible that
-                # the scheduler rescheduled this task, so kill launched processes.
-                # This can only really happen if the worker can't read the DB for a long time
-                time_since_last_heartbeat = (timezone.utcnow() - self.job.latest_heartbeat).total_seconds()
-                if time_since_last_heartbeat > heartbeat_time_limit:
-                    Stats.incr("local_task_job_prolonged_heartbeat_failure", 1, 1)
-                    self.log.error("Heartbeat time limit exceeded!")
-                    raise AirflowException(
-                        f"Time since last heartbeat({time_since_last_heartbeat:.2f}s) exceeded limit "
-                        f"({heartbeat_time_limit}s)."
                     )
+                    return_code = self.task_runner.return_code(timeout=max_wait_time)
+                    if return_code is not None:
+                        self.handle_task_exit(return_code)
+                        return return_code
+
+                    if span.is_recording():
+                        span.add_event(name="perform_heartbeat")
+                    try:
+                        perform_heartbeat(
+                            job=self.job, heartbeat_callback=self.heartbeat_callback, only_if_necessary=False
+                        )
+                    except Exception as e:
+                        # Failing the heartbeat should never kill the localtaskjob
+                        # If it repeatedly can't heartbeat, it will be marked as a zombie anyhow
+                        self.log.warning("Heartbeat failed with Exception: %s", e)
+
+                    # If it's been too long since we've heartbeat, then it's possible that
+                    # the scheduler rescheduled this task, so kill launched processes.
+                    # This can only really happen if the worker can't read the DB for a long time
+                    time_since_last_heartbeat = (
+                        timezone.utcnow() - self.job.latest_heartbeat
+                    ).total_seconds()
+                    if time_since_last_heartbeat > heartbeat_time_limit:
+                        Stats.incr("local_task_job_prolonged_heartbeat_failure", 1, 1)
+                        self.log.error("Heartbeat time limit exceeded!")
+                        if span.is_recording():
+                            span.add_event(
+                                name="error",
+                                attributes={
+                                    "message": "Heartbeat time limit exceeded",
+                                    "heartbeat_time_limit(s)": heartbeat_time_limit,
+                                    "time_since_last_heartbeat(s)": time_since_last_heartbeat,
+                                },
+                            )
+                        raise AirflowException(
+                            f"Time since last heartbeat({time_since_last_heartbeat:.2f}s) exceeded limit "
+                            f"({heartbeat_time_limit}s)."
+                        )
             return return_code
         finally:
             # Print a marker for log grouping of details before task execution
@@ -232,8 +255,8 @@ class LocalTaskJobRunner(BaseJobRunner, LoggingMixin):
         # Without setting this, heartbeat may get us
         self.terminating = True
         self._log_return_code_metric(return_code)
-        is_deferral = return_code == TaskReturnCode.DEFERRED.value
-        if is_deferral:
+
+        if is_deferral := return_code == TaskReturnCode.DEFERRED.value:
             self.log.info("Task exited with return code %s (task deferral)", return_code)
             _set_task_deferred_context_var()
         else:
@@ -277,7 +300,11 @@ class LocalTaskJobRunner(BaseJobRunner, LoggingMixin):
             recorded_pid = ti.pid
             same_process = recorded_pid == current_pid
 
-            if recorded_pid is not None and (ti.run_as_user or self.task_runner.run_as_user):
+            is_child_process = (ti.run_as_user and (ti.run_as_user != getuser())) or (
+                self.task_runner.run_as_user and (self.task_runner != getuser())
+            )
+
+            if recorded_pid is not None and is_child_process:
                 # when running as another user, compare the task runner pid to the parent of
                 # the recorded pid because user delegation becomes an extra process level.
                 # However, if recorded_pid is None, pass that through as it signals the task
@@ -293,6 +320,7 @@ class LocalTaskJobRunner(BaseJobRunner, LoggingMixin):
                 )
                 raise AirflowException("PID of job runner does not match")
         elif self.task_runner.return_code() is None and hasattr(self.task_runner, "process"):
+            self._overtime = (timezone.utcnow() - (ti.end_date or timezone.utcnow())).total_seconds()
             if ti.state == TaskInstanceState.SKIPPED:
                 # A DagRun timeout will cause tasks to be externally marked as skipped.
                 dagrun = ti.get_dagrun(session=session)
@@ -306,6 +334,11 @@ class LocalTaskJobRunner(BaseJobRunner, LoggingMixin):
                 if dagrun_timeout and execution_time > dagrun_timeout:
                     self.log.warning("DagRun timed out after %s.", execution_time)
 
+            # If process still runs after being marked as success, let it run until configured overtime
+            if ti.state == TaskInstanceState.SUCCESS and self._overtime < conf.getint(
+                "core", "task_success_overtime"
+            ):
+                return
             # potential race condition, the _run_raw_task commits `success` or other state
             # but task_runner does not exit right away due to slow process shutdown or any other reasons
             # let's do a throttle here, if the above case is true, the handle_task_exit will handle it
